/* This is the main program for the system call fuzz generator. */

#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>

#include "main.h"
#include "global.h"
#include "init.h"
#include "tests.h"

#if defined(INCLUDE_TIMER)
#  include "timer.h"
#endif


void main(int argc, char *argv[]) {
/***********************************************************************
Main program.
***********************************************************************/
# if defined(INCLUDE_TIMER)
    float time;
# endif

  get_options(argc, argv);

  process_options();

# if defined(INCLUDE_TIMER)
  start_timer();
# endif

  run_tests();

# if defined(INCLUDE_TIMER)
  time = end_timer();

  fprintf(stderr, "\n%d tests completed in %f seconds.\n",
		  NTest, time);
  fprintf(stderr, "Rates:  %g tests/second; %g msec/test.\n",
                  NTest/time, (time/NTest)*1000.0);
# endif

  if (SummaryFlag) write_summary_done();

  clean_up();

}


void clean_up() {
/***********************************************************************
Close files, etc.  (Assuming that the program ever gets here.)
***********************************************************************/
  if (LogFile) fclose(LogFile);
  if (SignalParent) kill(getppid(), SIGUSR1);
}


void write_summary_done(void) {
/***********************************************************************
Write the word "done" to the end of the summary file so that we know the
simulation finished.
***********************************************************************/

  assert(SummaryFlag);
  assert(SummaryFileName);

  /* Note:  The summary file is now left open so that it does not need
  to be opened here. */
  /* Open the file. */
  /*SummaryFile = fopen(SummaryFileName, "a");*/

  /*assert(SummaryFile);*/
  if (!SummaryFile) {
    fprintf(stderr, "Could not open the summary file!  ");
    fprintf(stderr, "errno = %d.\n", errno);
    exit(99);
  }

  fprintf(SummaryFile, "----- This test completed successfully. -----\n");

  fclose(SummaryFile);
  sync();

}


