/* main.h - Call the given (number) system call with the given
parameters */


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>


#define MAX_ARGUMENTS	10


/* Globals. */
int SysCallNum;
int NArg;
int Arg[MAX_ARGUMENTS];


/* Forward declarations. */
void get_args(int argc, char *argv[]);
int make_call(int sysCall, int nParm, int p[]);


void main(int argc, char *argv[]) {
/***********************************************************************
***********************************************************************/
  int result;

  /* Get the arguments from the command line. */
  get_args(argc, argv);

  /* Make the call. */
  result = make_call(SysCallNum, NArg, Arg);

  printf("The call returned %d.", result);
  if (result == -1) {
    printf("  The errno is %d.\n", errno);
  } else {
    printf("\n");
  }

}


void get_args(int argc, char *argv[]) {
/***********************************************************************
This routine gets the system call number and then assumes that all other
arguments are integer arguments to the system call.  It does no error
checking.
***********************************************************************/
  int num;

  num = 1;

  if (argc < 2) {
    fprintf(stderr, "Usage:  main <syscall number> [argument list]\n");
    exit(99);
  }

  /* Get the system call number. */
  SysCallNum = atoi(argv[num++]);

printf("syscall: %d:", SysCallNum);

  /* Get the arguments. */
  NArg = 0;
  for (; num < argc; num++) {

    if (NArg == MAX_ARGUMENTS) {
      fprintf(stderr, "Too many arguments given.\n");
      exit(98);
    }

    Arg[NArg++] = atoi(argv[num]);

printf(" %d", Arg[NArg-1]);

  }

printf("\n");


}


int make_call(int sysCall, int nParm, int p[]) {
/***********************************************************************
Make the specified system call with the specified number of parameters
and return the call's exit status.

Note:  Extra parameters are ignored, so this routine should just pass
the maximum number each time.  Is there any reason not to?
***********************************************************************/
  extern int syscall(int, ...);
  int result;

  switch (nParm) {
    case 0:
      result = syscall(sysCall);
      break;
    case 1:
      result = syscall(sysCall, p[0]);
      break;
    case 2:
      result = syscall(sysCall, p[0], p[1]);
      break;
    case 3:
      result = syscall(sysCall, p[0], p[1], p[2]);
      break;
    case 4:
      result = syscall(sysCall, p[0], p[1], p[2], p[3]);
      break;
    case 5:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4]);
      break;
    case 6:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5]);
      break;
    case 7:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5], p[6]);
      break;
    case 8:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5], p[6], p[7]);
      break;
    case 9:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5], p[6], p[7], p[8]);
      break;
    case 10:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5], p[6], p[7], p[8], p[9]);
      break;
  }

  return result;
}


