#include <stdio.h>
#include <errno.h>
#include <unistd.h>

/*#include <time.h>*/

#include "main.h"
#include "global.h"
#include "tests.h"


void run_tests(void) {
/***********************************************************************
Run the tests.

Notes:

Does the place in memory where the parameters are stored matter?  Will
certain locations cause a failure and others will not?

***********************************************************************/
  int test, n;
  /*int m;*/
  int sysCallN, sysCall, nParm;
  int exitStatus;

  /*struct timespec sleepTime;*/

  /*static int p[MAX_PARAMETERS];*/
  auto int p[MAX_PARAMETERS];

  printf("Starting the tests with random state:  %d %d %d\n",
         Rand48State[0], Rand48State[1], Rand48State[2]);

  printf("\n");
  fflush(stdout);

  for (test = 0; test < NTest; test++) {

    if (AnnounceTestsFlag && !PrintEachTestFlag) {

      /* Announce the test. */
      if ((test == 0) || (((test+1) % AnnounceTestsEach) == 0)) {
	printf("\rDoing test %d...", test+1);
	fflush(stdout);
      }
    }

    /* Choose the test. */
    sysCallN = get_syscall_index();
    sysCall = SysCall[sysCallN].num;

    /* Choose the number of parameters. */
    nParm = get_number_of_parameters();

    /* Choose the parameters. */
    get_parameters(sysCallN, p);

    /* Log the call. */
    if (LoggingFlag) {
      fprintf(LogFile, "#%d: %3d :%16.16s:",
              test+1, sysCall, SysCall[sysCallN].name+4);
      for (n = 0; n < nParm; n++) {
	if (AggressiveFlag) {
	  fprintf(LogFile, "%c%d",
	          SysCall[sysCallN].parmFlag[n] ? '*' : ' ', p[n]);
        } else {
	  fprintf(LogFile, " %d", p[n]);
	}
      }
      fprintf(LogFile, "\n");
      fflush(LogFile);

      /* sync() only schedules the write operation, while fsync()
      guarantees that it completes.  Thus, using fsync() eliminates the
      possibility that a crach occurs between the scheduling of writes
      and the actual write.
      
      Note:  This is true but unexpected, as the fsync() is faster than
      the sync() was.  Maybe it has something to do with the fact that
      fsync() works only on the file of interest. */


      /*sync();*/
      fsync(fileno(LogFile));
    }

    /* It is possible that the behavior of some systems may change when
    a well-defined system call is generated between each pair of random
    calls.  But then, maybe not... */

#   ifdef INSERT_INNOCUOUS_SYSCALL
      write(stdout->_file, ".", 1);
#   endif

    if (PrintEachTestFlag) {
      printf("Test %d is %3d (%16.16s) with %2d parms...",
             test+1, sysCall, SysCall[sysCallN].name+4, nParm);
      fflush(stdout);
    }

    /* Do the call. */
    exitStatus = make_call(sysCall, nParm, p);

    if (PrintEachTestFlag) {
      printf("exit %d(%d).\n", exitStatus, errno);
      fflush(stdout);
    }

  }

  if (AnnounceTestsFlag && !PrintEachTestFlag) {
    printf("done.\n");
  }

}


int get_syscall_index(void) {
/***********************************************************************
Return the index into the list of valid syscalls that corresponds to the
system call that should be tested.
***********************************************************************/
  int result;

  result = RAND(0, NSysCall-1);

  return result;
}


int get_number_of_parameters(void) {
/***********************************************************************
Return the number of parameters that should be sent to the system call.
***********************************************************************/
  int result;

  /*result = RAND(0, MaxParameters);*/
  result = 8;

  return result;
}


void get_parameters(int sysCallN, int p[]) {
/***********************************************************************
Set the values of the parameters to be passed.

Note:  This currently just filles the maximum number of parameters each
time it is called.  This may not be the best way to do it.  But, it
makes the parameters passed for each call the same, whether or not the
number of parameters being passed changes.

***********************************************************************/
  int n;
# if defined(INCLUDE_CODE_FOR_AGGRESSIVE_RANDOMIZATION)
  int m;
# endif

  /* Fill the random area that parameters can point to will random
  values.  This is done every time regardless of what parameters and
  options are used so that the actual sequence of calls will always be
  the same.  */

# if defined(INCLUDE_CODE_FOR_AGGRESSIVE_RANDOMIZATION)
  for (n = 0; n < MAX_PARAMETERS; n++) {
    for (m = 0; m < PARAMETER_SPACE_SIZE; m++) {
      ParameterSpace[n][m] = mrand48();
    }
  }
# endif

  /* Set up the parameters. */
  for (n = 0; n < MAX_PARAMETERS; n++) {
    /*if (AggressiveFlag && (SysCall[sysCallN].parmFlag[n])) {*/
    if (AggressiveFlag) {
      p[n] = (int)ParameterSpace[n];
      mrand48();   /* Dummy. */
    } else {
      p[n] = mrand48();
    }
  }

}


int make_call(int sysCall, int nParm, int p[]) {
/***********************************************************************
Make the specified system call with the specified number of parameters
and return the call's exit status.

Note:  Extra parameters are ignored, so this routine should just pass
the maximum number each time.  Is there any reason not to?

***********************************************************************/
  extern int syscall(int, ...);
  int result;

  switch (nParm) {
    case 0:
      result = syscall(sysCall);
      break;
    case 1:
      result = syscall(sysCall, p[0]);
      break;
    case 2:
      result = syscall(sysCall, p[0], p[1]);
      break;
    case 3:
      result = syscall(sysCall, p[0], p[1], p[2]);
      break;
    case 4:
      result = syscall(sysCall, p[0], p[1], p[2], p[3]);
      break;
    case 5:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4]);
      break;
    case 6:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5]);
      break;
    case 7:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5], p[6]);
      break;
    case 8:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5], p[6], p[7]);
      break;
    case 9:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5], p[6], p[7], p[8]);
      break;
    case 10:
      result = syscall(sysCall, p[0], p[1], p[2], p[3], p[4],
                                p[5], p[6], p[7], p[8], p[9]);
      break;
  }

  return result;
}


